@@ -1,4 +1,6 @@
 """Test for tmuxp command line interface."""
+import argparse
+import io
 import json
 import os
 import pathlib
@@ -6,15 +8,13 @@ import typing as t
 
 import pytest
 
-import click
-from click.testing import CliRunner
 from pytest_mock import MockerFixture
 
 import libtmux
 from libtmux.common import has_lt_version
 from libtmux.exc import LibTmuxException
+from libtmux.session import Session
 from tmuxp import cli, config, exc
-from tmuxp.cli.debug_info import command_debug_info
 from tmuxp.cli.import_config import get_teamocil_dir, get_tmuxinator_dir
 from tmuxp.cli.load import (
     _load_append_windows_to_current_session,
@@ -23,14 +23,13 @@ from tmuxp.cli.load import (
     load_plugins,
     load_workspace,
 )
-from tmuxp.cli.ls import command_ls
 from tmuxp.cli.utils import (
-    ConfigPath,
     _validate_choices,
     get_abs_path,
     get_config_dir,
     is_pure_name,
     scan_config,
+    tmuxp_echo,
 )
 from tmuxp.config_reader import ConfigReader
 from tmuxp.workspacebuilder import WorkspaceBuilder
@@ -39,17 +38,19 @@ from .constants import FIXTURE_PATH
 from .fixtures import utils as test_utils
 
 if t.TYPE_CHECKING:
+    import _pytest.capture
+
     from libtmux.server import Server
 
 
-def test_creates_config_dir_not_exists(tmp_path: pathlib.Path):
+def test_creates_config_dir_not_exists(tmp_path: pathlib.Path) -> None:
     """cli.startup() creates config dir if not exists."""
 
     cli.startup(tmp_path)
     assert os.path.exists(tmp_path)
 
 
-def test_in_dir_from_config_dir(tmp_path: pathlib.Path):
+def test_in_dir_from_config_dir(tmp_path: pathlib.Path) -> None:
     """config.in_dir() finds configs config dir."""
 
     cli.startup(tmp_path)
@@ -62,7 +63,7 @@ def test_in_dir_from_config_dir(tmp_path: pathlib.Path):
     assert len(configs_found) == 2
 
 
-def test_ignore_non_configs_from_current_dir(tmp_path: pathlib.Path):
+def test_ignore_non_configs_from_current_dir(tmp_path: pathlib.Path) -> None:
     """cli.in_dir() ignore non-config from config dir."""
 
     cli.startup(tmp_path)
@@ -75,7 +76,9 @@ def test_ignore_non_configs_from_current_dir(tmp_path: pathlib.Path):
     assert len(configs_found) == 1
 
 
-def test_get_configs_cwd(tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch):
+def test_get_configs_cwd(
+    tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch
+) -> None:
     """config.in_cwd() find config in shell current working directory."""
 
     confdir = tmp_path / "tmuxpconf2"
@@ -106,7 +109,7 @@ def test_get_configs_cwd(tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch
         ("myproject", True),
     ],
 )
-def test_is_pure_name(path, expect):
+def test_is_pure_name(path: str, expect: bool) -> None:
     assert is_pure_name(path) == expect
 
 
@@ -130,33 +133,37 @@ def test_is_pure_name(path, expect):
 
 
 @pytest.fixture
-def homedir(tmp_path: pathlib.Path):
+def homedir(tmp_path: pathlib.Path) -> pathlib.Path:
     home = tmp_path / "home"
     home.mkdir()
     return home
 
 
 @pytest.fixture
-def configdir(homedir):
+def configdir(homedir: pathlib.Path) -> pathlib.Path:
     conf = homedir / ".tmuxp"
     conf.mkdir()
     return conf
 
 
 @pytest.fixture
-def projectdir(homedir):
+def projectdir(homedir: pathlib.Path) -> pathlib.Path:
     proj = homedir / "work" / "project"
     proj.mkdir(parents=True)
     return proj
 
 
-def test_tmuxp_configdir_env_var(tmp_path: pathlib.Path, monkeypatch):
+def test_tmuxp_configdir_env_var(
+    tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setenv("TMUXP_CONFIGDIR", str(tmp_path))
 
     assert get_config_dir() == str(tmp_path)
 
 
-def test_tmuxp_configdir_xdg_config_dir(tmp_path: pathlib.Path, monkeypatch):
+def test_tmuxp_configdir_xdg_config_dir(
+    tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path))
     tmux_dir = tmp_path / "tmuxp"
     tmux_dir.mkdir()
@@ -170,7 +177,7 @@ def test_resolve_dot(
     configdir: pathlib.Path,
     projectdir: pathlib.Path,
     monkeypatch: pytest.MonkeyPatch,
-):
+) -> None:
     monkeypatch.setenv("HOME", str(homedir))
     monkeypatch.setenv("XDG_CONFIG_HOME", str(homedir / ".config"))
 
@@ -272,14 +279,17 @@ def test_resolve_dot(
 
 
 def test_scan_config_arg(
-    homedir, configdir, projectdir, monkeypatch: pytest.MonkeyPatch
-):
-    runner = CliRunner()
+    homedir: pathlib.Path,
+    configdir: pathlib.Path,
+    projectdir: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
+    parser = argparse.ArgumentParser()
+    parser.add_argument("config_file", type=str)
 
-    @click.command()
-    @click.argument("config", type=ConfigPath(exists=True), nargs=-1)
-    def config_cmd(config):
-        click.echo(config)
+    def config_cmd(config_file: str) -> None:
+        tmuxp_echo(scan_config(config_file, config_dir=configdir))
 
     monkeypatch.setenv("HOME", str(homedir))
     tmuxp_config_path = projectdir / ".tmuxp.yaml"
@@ -290,28 +300,34 @@ def test_scan_config_arg(
 
     project_config = projectdir / ".tmuxp.yaml"
 
-    def check_cmd(config_arg):
-        return runner.invoke(config_cmd, [config_arg]).output
+    def check_cmd(config_arg) -> "_pytest.capture.CaptureResult":
+        args = parser.parse_args([config_arg])
+        config_cmd(config_file=args.config_file)
+        return capsys.readouterr()
 
     monkeypatch.chdir(projectdir)
     expect = str(project_config)
-    assert expect in check_cmd(".")
-    assert expect in check_cmd("./")
-    assert expect in check_cmd("")
-    assert expect in check_cmd("../project")
-    assert expect in check_cmd("../project/")
-    assert expect in check_cmd(".tmuxp.yaml")
-    assert str(user_config) in check_cmd("../../.tmuxp/%s.yaml" % user_config_name)
-    assert user_config.stem in check_cmd("myconfig")
-    assert str(user_config) in check_cmd("~/.tmuxp/myconfig.yaml")
+    assert expect in check_cmd(".").out
+    assert expect in check_cmd("./").out
+    assert expect in check_cmd("").out
+    assert expect in check_cmd("../project").out
+    assert expect in check_cmd("../project/").out
+    assert expect in check_cmd(".tmuxp.yaml").out
+    assert str(user_config) in check_cmd("../../.tmuxp/%s.yaml" % user_config_name).out
+    assert user_config.stem in check_cmd("myconfig").out
+    assert str(user_config) in check_cmd("~/.tmuxp/myconfig.yaml").out
 
-    assert "file not found" in check_cmd(".tmuxp.json")
-    assert "file not found" in check_cmd(".tmuxp.ini")
-    assert "No tmuxp files found" in check_cmd("../")
-    assert "config not found in config dir" in check_cmd("moo")
+    with pytest.raises(FileNotFoundError, match="file not found"):
+        assert "file not found" in check_cmd(".tmuxp.json").err
+    with pytest.raises(FileNotFoundError, match="file not found"):
+        assert "file not found" in check_cmd(".tmuxp.ini").err
+    with pytest.raises(FileNotFoundError, match="No tmuxp files found"):
+        assert "No tmuxp files found" in check_cmd("../").err
+    with pytest.raises(FileNotFoundError, match="config not found in config dir"):
+        assert "config not found in config dir" in check_cmd("moo").err
 
 
-def test_load_workspace(server, monkeypatch):
+def test_load_workspace(server: "Server", monkeypatch: pytest.MonkeyPatch) -> None:
     # this is an implementation test. Since this testsuite may be ran within
     # a tmux session by the developer himself, delete the TMUX variable
     # temporarily.
@@ -323,11 +339,13 @@ def test_load_workspace(server, monkeypatch):
         session_file, socket_name=server.socket_name, detached=True
     )
 
-    assert isinstance(session, libtmux.Session)
+    assert isinstance(session, Session)
     assert session.name == "sampleconfig"
 
 
-def test_load_workspace_named_session(server, monkeypatch):
+def test_load_workspace_named_session(
+    server: "Server", monkeypatch: pytest.MonkeyPatch
+) -> None:
     # this is an implementation test. Since this testsuite may be ran within
     # a tmux session by the developer himself, delete the TMUX variable
     # temporarily.
@@ -342,7 +360,7 @@ def test_load_workspace_named_session(server, monkeypatch):
         detached=True,
     )
 
-    assert isinstance(session, libtmux.Session)
+    assert isinstance(session, Session)
     assert session.name == "tmuxp-new"
 
 
@@ -350,8 +368,8 @@ def test_load_workspace_named_session(server, monkeypatch):
     has_lt_version("2.1"), reason="exact session name matches only tmux >= 2.1"
 )
 def test_load_workspace_name_match_regression_252(
-    tmp_path: pathlib.Path, server, monkeypatch
-):
+    tmp_path: pathlib.Path, server: "Server", monkeypatch: pytest.MonkeyPatch
+) -> None:
     monkeypatch.delenv("TMUX", raising=False)
     session_file = FIXTURE_PATH / "workspacebuilder" / "two_pane.yaml"
 
@@ -360,7 +378,7 @@ def test_load_workspace_name_match_regression_252(
         session_file, socket_name=server.socket_name, detached=True
     )
 
-    assert isinstance(session, libtmux.Session)
+    assert isinstance(session, Session)
     assert session.name == "sampleconfig"
 
     projfile = tmp_path / "simple.yaml"
@@ -379,10 +397,13 @@ windows:
     session = load_workspace(
         str(projfile), socket_name=server.socket_name, detached=True
     )
+    assert session is not None
     assert session.name == "sampleconfi"
 
 
-def test_load_symlinked_workspace(server, tmp_path, monkeypatch):
+def test_load_symlinked_workspace(
+    server: "Server", tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch
+) -> None:
     # this is an implementation test. Since this testsuite may be ran within
     # a tmux session by the developer himself, delete the TMUX variable
     # temporarily.
@@ -408,70 +429,103 @@ windows:
     session = load_workspace(
         str(projfile), socket_name=server.socket_name, detached=True
     )
+    assert session is not None
+    assert session.attached_window is not None
     pane = session.attached_window.attached_pane
 
-    assert isinstance(session, libtmux.Session)
+    assert isinstance(session, Session)
     assert session.name == "samplesimple"
+
+    assert pane is not None
     assert pane.current_path == str(realtemp)
 
 
 def test_regression_00132_session_name_with_dots(
-    tmp_path: pathlib.Path, server, session
-):
+    tmp_path: pathlib.Path,
+    server: "Server",
+    session: Session,
+    capsys: pytest.CaptureFixture,
+) -> None:
     yaml_config = FIXTURE_PATH / "workspacebuilder" / "regression_00132_dots.yaml"
     cli_args = [str(yaml_config)]
-    inputs: t.List[str] = []
-    runner = CliRunner()
-    result = runner.invoke(
-        cli.command_load, cli_args, input="".join(inputs), standalone_mode=False
-    )
-    assert result.exception
-    assert isinstance(result.exception, libtmux.exc.BadSessionName)
+    with pytest.raises(libtmux.exc.BadSessionName):
+        cli.cli(["load", *cli_args])
 
 
-@pytest.mark.parametrize("cli_args", [(["load", "."]), (["load", ".tmuxp.yaml"])])
-def test_load_zsh_autotitle_warning(cli_args, tmp_path, monkeypatch):
+@pytest.mark.parametrize(
+    "cli_args", [["load", ".", "-d"], ["load", ".tmuxp.yaml", "-d"]]
+)
+def test_load_zsh_autotitle_warning(
+    cli_args: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+    server: "Server",
+) -> None:
     # create dummy tmuxp yaml so we don't get yelled at
     yaml_config = tmp_path / ".tmuxp.yaml"
-    yaml_config.touch()
+    yaml_config.write_text(
+        """
+    session_name: test
+    windows:
+    - window_name: test
+      panes:
+      -
+    """,
+        encoding="utf-8",
+    )
     oh_my_zsh_path = tmp_path / ".oh-my-zsh"
     oh_my_zsh_path.mkdir()
     monkeypatch.setenv("HOME", str(tmp_path))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
 
     monkeypatch.delenv("DISABLE_AUTO_TITLE", raising=False)
     monkeypatch.setenv("SHELL", "zsh")
-    result = runner.invoke(cli.cli, cli_args)
-    assert "Please set" in result.output
+
+    # Use tmux server (socket name) used in the test
+    assert server.socket_name is not None
+    cli_args = cli_args + ["-L", server.socket_name]
+
+    cli.cli(cli_args)
+    result = capsys.readouterr()
+    assert "Please set" in result.out
 
     monkeypatch.setenv("DISABLE_AUTO_TITLE", "false")
-    result = runner.invoke(cli.cli, cli_args)
-    assert "Please set" in result.output
+    cli.cli(cli_args)
+    result = capsys.readouterr()
+    assert "Please set" in result.out
 
     monkeypatch.setenv("DISABLE_AUTO_TITLE", "true")
-    result = runner.invoke(cli.cli, cli_args)
-    assert "Please set" not in result.output
+    cli.cli(cli_args)
+    result = capsys.readouterr()
+    assert "Please set" not in result.out
 
     monkeypatch.delenv("DISABLE_AUTO_TITLE", raising=False)
     monkeypatch.setenv("SHELL", "sh")
-    result = runner.invoke(cli.cli, cli_args)
-    assert "Please set" not in result.output
+    cli.cli(cli_args)
+    result = capsys.readouterr()
+    assert "Please set" not in result.out
 
 
 @pytest.mark.parametrize(
     "cli_args",
     [
-        (["load", ".", "--log-file", "log.txt"]),
+        (["load", ".", "--log-file", "log.txt", "-d"]),
     ],
 )
-def test_load_log_file(cli_args, tmp_path, monkeypatch):
+def test_load_log_file(
+    cli_args: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
     # create dummy tmuxp yaml that breaks to prevent actually loading tmux
     tmuxp_config_path = tmp_path / ".tmuxp.yaml"
     tmuxp_config_path.write_text(
         """
 session_name: hello
+  -
         """,
         encoding="utf-8",
     )
@@ -480,18 +534,18 @@ session_name: hello
     monkeypatch.setenv("HOME", str(tmp_path))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
 
-    # If autoconfirm (-y) no need to prompt y
-    input_args = "y\ny\n" if "-y" not in cli_args else ""
-
-    result = runner.invoke(cli.cli, cli_args, input=input_args)
+    try:
+        cli.cli(cli_args)
+    except Exception:
+        pass
+    result = capsys.readouterr()
     log_file_path = tmp_path / "log.txt"
     assert "Loading" in log_file_path.open().read()
-    assert result is not None
+    assert result.out is not None
 
 
-@pytest.mark.parametrize("cli_cmd", ["shell", ("shell", "--pdb")])
+@pytest.mark.parametrize("cli_cmd", [["shell"], ["shell", "--pdb"]])
 @pytest.mark.parametrize(
     "cli_args,inputs,env,expected_output",
     [
@@ -561,50 +615,49 @@ session_name: hello
     ],
 )
 def test_shell(
-    cli_cmd,
-    cli_args,
-    inputs,
-    expected_output,
-    env,
-    tmp_path,
-    monkeypatch,
-    server,
-    session,
-):
+    cli_cmd: t.List[str],
+    cli_args: t.List[str],
+    inputs: t.List[t.Any],
+    expected_output: str,
+    env: t.Dict[str, str],
+    server: "Server",
+    session: Session,
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
     monkeypatch.setenv("HOME", str(tmp_path))
     window_name = "my_window"
     window = session.new_window(window_name=window_name)
     window.split_window()
 
+    assert window.attached_pane is not None
+
     template_ctx = dict(
         SOCKET_NAME=server.socket_name,
-        SOCKET_PATH=server.socket_path,
         SESSION_NAME=session.name,
         WINDOW_NAME=window_name,
         PANE_ID=window.attached_pane.id,
         SERVER_SOCKET_NAME=server.socket_name,
     )
 
-    cli_cmd = list(cli_cmd) if isinstance(cli_cmd, (list, tuple)) else [cli_cmd]
     cli_args = cli_cmd + [cli_arg.format(**template_ctx) for cli_arg in cli_args]
 
     for k, v in env.items():
         monkeypatch.setenv(k, v.format(**template_ctx))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
 
-    result = runner.invoke(
-        cli.cli, cli_args, input="".join(inputs), catch_exceptions=False
-    )
-    assert expected_output.format(**template_ctx) in result.output
+    cli.cli(cli_args)
+    result = capsys.readouterr()
+    assert expected_output.format(**template_ctx) in result.out
 
 
 @pytest.mark.parametrize(
     "cli_cmd",
     [
-        "shell",
-        ("shell", "--pdb"),
+        ["shell"],
+        ["shell", "--pdb"],
     ],
 )
 @pytest.mark.parametrize(
@@ -648,63 +701,61 @@ def test_shell(
     ],
 )
 def test_shell_target_missing(
-    cli_cmd,
-    cli_args,
-    inputs,
-    env,
-    template_ctx,
-    exception,
-    message,
-    tmp_path,
-    monkeypatch,
-    socket_name,
-    server,
-    session,
-):
+    cli_cmd: t.List[str],
+    cli_args: t.List[str],
+    inputs: t.List[t.Any],
+    env: t.Dict[t.Any, t.Any],
+    template_ctx: t.Dict[str, str],
+    exception: t.Union[t.Type[exc.TmuxpException], t.Type[LibTmuxException]],
+    message: str,
+    socket_name: str,
+    server: "Server",
+    session: Session,
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
     monkeypatch.setenv("HOME", str(tmp_path))
     window_name = "my_window"
     window = session.new_window(window_name=window_name)
     window.split_window()
 
-    template_ctx = dict(
-        SOCKET_NAME=server.socket_name,
-        SOCKET_PATH=server.socket_path,
-        SESSION_NAME=session.name,
-        WINDOW_NAME=template_ctx.get("window_name", window_name),
-        PANE_ID=template_ctx.get("pane_id"),
-        SERVER_SOCKET_NAME=server.socket_name,
+    assert server.socket_name is not None
+    assert session.name is not None
+
+    template_ctx.update(
+        dict(
+            SOCKET_NAME=server.socket_name,
+            SESSION_NAME=session.name,
+            WINDOW_NAME=template_ctx.get("window_name", window_name),
+        )
     )
-    cli_cmd = list(cli_cmd) if isinstance(cli_cmd, (list, tuple)) else [cli_cmd]
     cli_args = cli_cmd + [cli_arg.format(**template_ctx) for cli_arg in cli_args]
 
     for k, v in env.items():
         monkeypatch.setenv(k, v.format(**template_ctx))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
 
     if exception is not None:
         with pytest.raises(exception, match=message.format(**template_ctx)):
-            result = runner.invoke(
-                cli.cli, cli_args, input="".join(inputs), catch_exceptions=False
-            )
+            cli.cli(cli_args)
     else:
-        result = runner.invoke(
-            cli.cli, cli_args, input="".join(inputs), catch_exceptions=False
-        )
-        assert message.format(**template_ctx) in result.output
+        cli.cli(cli_args)
+        result = capsys.readouterr()
+        assert message.format(**template_ctx) in result.out
 
 
 @pytest.mark.parametrize(
     "cli_cmd",
     [
-        # 'shell',
-        # ('shell', '--pdb'),
-        ("shell", "--code"),
-        # ('shell', '--bpython'),
-        # ('shell', '--ptipython'),
-        # ('shell', '--ptpython'),
-        # ('shell', '--ipython'),
+        # ['shell'],
+        # ['shell', '--pdb'),
+        ["shell", "--code"],
+        # ['shell', '--bpython'],
+        # ['shell', '--ptipython'],
+        # ['shell', '--ptpython'],
+        # ['shell', '--ipython'],
     ],
 )
 @pytest.mark.parametrize(
@@ -728,44 +779,46 @@ def test_shell_target_missing(
         ),
     ],
 )
-def test_shell_plus(
-    cli_cmd,
-    cli_args,
-    inputs,
-    env,
-    message,
-    tmp_path,
-    monkeypatch,
-    server,
-    session,
-):
+def test_shell_interactive(
+    cli_cmd: t.List[str],
+    cli_args: t.List[str],
+    inputs: t.List[t.Any],
+    env: t.Dict[str, str],
+    message: str,
+    server: "Server",
+    session: Session,
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
     monkeypatch.setenv("HOME", str(tmp_path))
     window_name = "my_window"
     window = session.new_window(window_name=window_name)
     window.split_window()
 
+    assert window.attached_pane is not None
+
     template_ctx = dict(
         SOCKET_NAME=server.socket_name,
-        SOCKET_PATH=server.socket_path,
         SESSION_NAME=session.name,
         WINDOW_NAME=window_name,
         PANE_ID=window.attached_pane.id,
         SERVER_SOCKET_NAME=server.socket_name,
     )
 
-    cli_cmd = list(cli_cmd) if isinstance(cli_cmd, (list, tuple)) else [cli_cmd]
     cli_args = cli_cmd + [cli_arg.format(**template_ctx) for cli_arg in cli_args]
 
     for k, v in env.items():
         monkeypatch.setenv(k, v.format(**template_ctx))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
-
-    result = runner.invoke(
-        cli.cli, cli_args, input="".join(inputs), catch_exceptions=True
-    )
-    assert message.format(**template_ctx) in result.output
+    monkeypatch.setattr("sys.stdin", io.StringIO("exit()\r"))
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+    result = capsys.readouterr()
+    assert message.format(**template_ctx) in result.err
 
 
 @pytest.mark.parametrize(
@@ -778,7 +831,11 @@ def test_shell_plus(
         (["convert", ".tmuxp.yml", "-y"]),
     ],
 )
-def test_convert(cli_args, tmp_path, monkeypatch):
+def test_convert(
+    cli_args: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     # create dummy tmuxp yaml so we don't get yelled at
     filename = cli_args[1]
     if filename == ".":
@@ -792,12 +849,15 @@ def test_convert(cli_args, tmp_path, monkeypatch):
     monkeypatch.setenv("HOME", str(tmp_path))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
 
     # If autoconfirm (-y) no need to prompt y
     input_args = "y\ny\n" if "-y" not in cli_args else ""
 
-    runner.invoke(cli.cli, cli_args, input=input_args)
+    monkeypatch.setattr("sys.stdin", io.StringIO(input_args))
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
     tmuxp_json = tmp_path / ".tmuxp.json"
     assert tmuxp_json.exists()
     assert tmuxp_json.open().read() == json.dumps({"session_name": "hello"}, indent=2)
@@ -811,7 +871,11 @@ def test_convert(cli_args, tmp_path, monkeypatch):
         (["convert", ".tmuxp.json", "-y"]),
     ],
 )
-def test_convert_json(cli_args, tmp_path, monkeypatch):
+def test_convert_json(
+    cli_args: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     # create dummy tmuxp yaml so we don't get yelled at
     json_config = tmp_path / ".tmuxp.json"
     json_config.write_text('{"session_name": "hello"}', encoding="utf-8")
@@ -820,24 +884,32 @@ def test_convert_json(cli_args, tmp_path, monkeypatch):
     monkeypatch.setenv("HOME", str(tmp_path))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
 
     # If autoconfirm (-y) no need to prompt y
     input_args = "y\ny\n" if "-y" not in cli_args else ""
 
-    runner.invoke(cli.cli, cli_args, input=input_args)
+    monkeypatch.setattr("sys.stdin", io.StringIO(input_args))
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+
     tmuxp_yaml = tmp_path / ".tmuxp.yaml"
     assert tmuxp_yaml.exists()
     assert tmuxp_yaml.open().read() == "session_name: hello\n"
 
 
 @pytest.mark.parametrize("cli_args", [(["import"])])
-def test_import(cli_args, monkeypatch):
-    runner = CliRunner()
-
-    result = runner.invoke(cli.cli, cli_args)
-    assert "tmuxinator" in result.output
-    assert "teamocil" in result.output
+def test_import(
+    cli_args: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
+    cli.cli(cli_args)
+    result = capsys.readouterr()
+    assert "tmuxinator" in result.out
+    assert "teamocil" in result.out
 
 
 @pytest.mark.parametrize(
@@ -847,11 +919,19 @@ def test_import(cli_args, monkeypatch):
         (["-h"]),
     ],
 )
-def test_help(cli_args, monkeypatch):
-    runner = CliRunner()
+def test_help(
+    cli_args: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+    result = capsys.readouterr()
 
-    result = runner.invoke(cli.cli, cli_args)
-    assert "Usage: cli [OPTIONS] COMMAND [ARGS]..." in result.output
+    assert "usage: tmuxp [-h] [--version] [--log-level LOG_LEVEL]" in result.out
 
 
 @pytest.mark.parametrize(
@@ -871,7 +951,12 @@ def test_help(cli_args, monkeypatch):
         ),
     ],
 )
-def test_import_teamocil(cli_args, inputs, tmp_path, monkeypatch):
+def test_import_teamocil(
+    cli_args: t.List[str],
+    inputs: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     teamocil_config = test_utils.read_config_file("config_teamocil/test4.yaml")
 
     teamocil_path = tmp_path / ".teamocil"
@@ -886,8 +971,12 @@ def test_import_teamocil(cli_args, inputs, tmp_path, monkeypatch):
     monkeypatch.setenv("HOME", str(tmp_path))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
-    runner.invoke(cli.cli, cli_args, input="".join(inputs))
+    monkeypatch.setattr("sys.stdin", io.StringIO("".join(inputs)))
+
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
 
     new_config_yaml = tmp_path / "la.yaml"
     assert new_config_yaml.exists()
@@ -910,7 +999,12 @@ def test_import_teamocil(cli_args, inputs, tmp_path, monkeypatch):
         ),
     ],
 )
-def test_import_tmuxinator(cli_args, inputs, tmp_path, monkeypatch):
+def test_import_tmuxinator(
+    cli_args: t.List[str],
+    inputs: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     tmuxinator_config = test_utils.read_config_file("config_tmuxinator/test3.yaml")
 
     tmuxinator_path = tmp_path / ".tmuxinator"
@@ -925,9 +1019,13 @@ def test_import_tmuxinator(cli_args, inputs, tmp_path, monkeypatch):
     monkeypatch.setenv("HOME", str(tmp_path))
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
-    out = runner.invoke(cli.cli, cli_args, input="".join(inputs))
-    print(out.output)
+
+    monkeypatch.setattr("sys.stdin", io.StringIO("".join(inputs)))
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+
     new_config_yaml = tmp_path / "la.yaml"
     assert new_config_yaml.exists()
 
@@ -947,7 +1045,13 @@ def test_import_tmuxinator(cli_args, inputs, tmp_path, monkeypatch):
         (["freeze"], ["y\n", "./exists.yaml\n", "./la.yaml\n", "y\n"]),  # Exists
     ],
 )
-def test_freeze(server, cli_args, inputs, tmp_path, monkeypatch):
+def test_freeze(
+    server: "Server",
+    cli_args: t.List[str],
+    inputs: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     monkeypatch.setenv("HOME", str(tmp_path))
     exists_yaml = tmp_path / "exists.yaml"
     exists_yaml.touch()
@@ -963,11 +1067,15 @@ def test_freeze(server, cli_args, inputs, tmp_path, monkeypatch):
     monkeypatch.setenv("TMUX_PANE", first_pane_on_second_session_id)
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
     # Use tmux server (socket name) used in the test
+    assert server.socket_name is not None
     cli_args = cli_args + ["-L", server.socket_name]
-    out = runner.invoke(cli.cli, cli_args, input="".join(inputs))
-    print(out.output)
+
+    monkeypatch.setattr("sys.stdin", io.StringIO("".join(inputs)))
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
 
     yaml_config_path = tmp_path / "la.yaml"
     assert yaml_config_path.exists()
@@ -983,15 +1091,21 @@ def test_freeze(server, cli_args, inputs, tmp_path, monkeypatch):
     [
         (  # Overwrite
             ["freeze", "mysession", "--force"],
-            ["\n", "y\n", "./exists.yaml\n", "y\n"],
+            ["\n", "\n", "y\n", "./exists.yaml\n", "y\n"],
         ),
         (  # Imply current session if not entered
             ["freeze", "--force"],
-            ["\n", "y\n", "./exists.yaml\n", "y\n"],
+            ["\n", "\n", "y\n", "./exists.yaml\n", "y\n"],
         ),
     ],
 )
-def test_freeze_overwrite(server, cli_args, inputs, tmp_path, monkeypatch):
+def test_freeze_overwrite(
+    server: "Server",
+    cli_args: t.List[str],
+    inputs: t.List[str],
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     monkeypatch.setenv("HOME", str(tmp_path))
     exists_yaml = tmp_path / "exists.yaml"
     exists_yaml.touch()
@@ -999,17 +1113,21 @@ def test_freeze_overwrite(server, cli_args, inputs, tmp_path, monkeypatch):
     server.new_session(session_name="mysession")
 
     monkeypatch.chdir(tmp_path)
-    runner = CliRunner()
     # Use tmux server (socket name) used in the test
+    assert server.socket_name is not None
     cli_args = cli_args + ["-L", server.socket_name]
-    out = runner.invoke(cli.cli, cli_args, input="".join(inputs))
-    print(out.output)
+
+    monkeypatch.setattr("sys.stdin", io.StringIO("".join(inputs)))
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
 
     yaml_config_path = tmp_path / "exists.yaml"
     assert yaml_config_path.exists()
 
 
-def test_get_abs_path(tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch):
+def test_get_abs_path(tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch) -> None:
     expect = str(tmp_path)
     monkeypatch.chdir(tmp_path)
     get_abs_path("../") == os.path.dirname(expect)
@@ -1018,7 +1136,7 @@ def test_get_abs_path(tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch):
     get_abs_path(expect) == expect
 
 
-def test_get_tmuxinator_dir(monkeypatch):
+def test_get_tmuxinator_dir(monkeypatch: pytest.MonkeyPatch) -> None:
     assert get_tmuxinator_dir() == os.path.expanduser("~/.tmuxinator/")
 
     monkeypatch.setenv("HOME", "/moo")
@@ -1026,7 +1144,7 @@ def test_get_tmuxinator_dir(monkeypatch):
     assert get_tmuxinator_dir() == os.path.expanduser("~/.tmuxinator/")
 
 
-def test_get_teamocil_dir(monkeypatch: pytest.MonkeyPatch):
+def test_get_teamocil_dir(monkeypatch: pytest.MonkeyPatch) -> None:
     assert get_teamocil_dir() == os.path.expanduser("~/.teamocil/")
 
     monkeypatch.setenv("HOME", "/moo")
@@ -1034,19 +1152,21 @@ def test_get_teamocil_dir(monkeypatch: pytest.MonkeyPatch):
     assert get_teamocil_dir() == os.path.expanduser("~/.teamocil/")
 
 
-def test_validate_choices():
+def test_validate_choices() -> None:
     validate = _validate_choices(["choice1", "choice2"])
 
     assert validate("choice1")
     assert validate("choice2")
 
-    with pytest.raises(click.BadParameter):
+    with pytest.raises(ValueError):
         assert validate("choice3")
 
 
 def test_pass_config_dir_ClickPath(
-    tmp_path: pathlib.Path, monkeypatch: pytest.MonkeyPatch
-):
+    tmp_path: pathlib.Path,
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
 
     configdir = tmp_path / "myconfigdir"
     configdir.mkdir()
@@ -1056,31 +1176,33 @@ def test_pass_config_dir_ClickPath(
 
     expect = str(user_config)
 
-    runner = CliRunner()
+    parser = argparse.ArgumentParser()
+    parser.add_argument("config_file", type=str)
 
-    @click.command()
-    @click.argument(
-        "config",
-        type=ConfigPath(exists=True, config_dir=(str(configdir))),
-        nargs=-1,
-    )
-    def config_cmd(config):
-        click.echo(config)
+    def config_cmd(config_file: str) -> None:
+        tmuxp_echo(scan_config(config_file, config_dir=configdir))
 
-    def check_cmd(config_arg):
-        return runner.invoke(config_cmd, [config_arg]).output
+    def check_cmd(config_arg) -> "_pytest.capture.CaptureResult":
+        args = parser.parse_args([config_arg])
+        config_cmd(config_file=args.config_file)
+        return capsys.readouterr()
 
     monkeypatch.chdir(configdir)
 
-    assert expect in check_cmd("myconfig")
-    assert expect in check_cmd("myconfig.yaml")
-    assert expect in check_cmd("./myconfig.yaml")
-    assert str(user_config) in check_cmd(str(configdir / "myconfig.yaml"))
+    assert expect in check_cmd("myconfig").out
+    assert expect in check_cmd("myconfig.yaml").out
+    assert expect in check_cmd("./myconfig.yaml").out
+    assert str(user_config) in check_cmd(str(configdir / "myconfig.yaml")).out
 
-    assert "file not found" in check_cmd(".tmuxp.json")
+    with pytest.raises(FileNotFoundError):
+        assert "FileNotFoundError" in check_cmd(".tmuxp.json").out
 
 
-def test_ls_cli(monkeypatch: pytest.MonkeyPatch, tmp_path: pathlib.Path):
+def test_ls_cli(
+    monkeypatch: pytest.MonkeyPatch,
+    tmp_path: pathlib.Path,
+    capsys: pytest.CaptureFixture,
+) -> None:
     monkeypatch.setenv("HOME", str(tmp_path))
     monkeypatch.setenv("XDG_CONFIG_HOME", str(tmp_path / ".config"))
 
@@ -1106,12 +1228,16 @@ def test_ls_cli(monkeypatch: pytest.MonkeyPatch, tmp_path: pathlib.Path):
         else:
             location.touch()
 
-    runner = CliRunner()
-    cli_output = runner.invoke(command_ls).output
+    try:
+        cli.cli(["ls"])
+    except SystemExit:
+        pass
+    cli_output = capsys.readouterr().out
+
     assert cli_output == "\n".join(stems) + "\n"
 
 
-def test_load_plugins(monkeypatch_plugin_test_packages):
+def test_load_plugins(monkeypatch_plugin_test_packages: None) -> None:
     from tmuxp_test_plugin_bwb.plugin import PluginBeforeWorkspaceBuilder
 
     plugins_config = test_utils.read_config_file("workspacebuilder/plugin_bwb.yaml")
@@ -1141,12 +1267,15 @@ def test_load_plugins(monkeypatch_plugin_test_packages):
     ],
 )
 def test_load_plugins_version_fail_skip(
-    monkeypatch_plugin_test_packages, cli_args, inputs
-):
-    runner = CliRunner()
+    monkeypatch_plugin_test_packages, cli_args, inputs, capsys: pytest.CaptureFixture
+) -> None:
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+    result = capsys.readouterr()
 
-    results = runner.invoke(cli.cli, cli_args, input="".join(inputs))
-    assert "[Loading]" in results.output
+    assert "[Loading]" in result.out
 
 
 @pytest.mark.parametrize(
@@ -1159,27 +1288,45 @@ def test_load_plugins_version_fail_skip(
     ],
 )
 def test_load_plugins_version_fail_no_skip(
-    monkeypatch_plugin_test_packages, cli_args, inputs
-):
-    runner = CliRunner()
+    monkeypatch_plugin_test_packages: None,
+    cli_args: t.List[str],
+    inputs: t.List[str],
+    monkeypatch: pytest.MonkeyPatch,
+    capsys: pytest.CaptureFixture,
+) -> None:
+    monkeypatch.setattr("sys.stdin", io.StringIO("".join(inputs)))
 
-    results = runner.invoke(cli.cli, cli_args, input="".join(inputs))
-    assert "[Not Skipping]" in results.output
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+    result = capsys.readouterr()
+
+    assert "[Not Skipping]" in result.out
 
 
 @pytest.mark.parametrize(
     "cli_args", [(["load", "tests/fixtures/workspacebuilder/plugin_missing_fail.yaml"])]
 )
-def test_load_plugins_plugin_missing(monkeypatch_plugin_test_packages, cli_args):
-    runner = CliRunner()
+def test_load_plugins_plugin_missing(
+    monkeypatch_plugin_test_packages: None,
+    cli_args: t.List[str],
+    capsys: pytest.CaptureFixture,
+) -> None:
+    try:
+        cli.cli(cli_args)
+    except SystemExit:
+        pass
+    result = capsys.readouterr()
 
-    results = runner.invoke(cli.cli, cli_args)
-    assert "[Plugin Error]" in results.output
+    assert "[Plugin Error]" in result.out
 
 
 def test_plugin_system_before_script(
-    monkeypatch_plugin_test_packages, server, monkeypatch
-):
+    monkeypatch_plugin_test_packages: None,
+    server: "Server",
+    monkeypatch: pytest.MonkeyPatch,
+) -> None:
     # this is an implementation test. Since this testsuite may be ran within
     # a tmux session by the developer himself, delete the TMUX variable
     # temporarily.
@@ -1195,7 +1342,9 @@ def test_plugin_system_before_script(
     assert session.name == "plugin_test_bs"
 
 
-def test_reattach_plugins(monkeypatch_plugin_test_packages, server):
+def test_reattach_plugins(
+    monkeypatch_plugin_test_packages: None, server: "Server"
+) -> None:
     config_plugins = test_utils.read_config_file("workspacebuilder/plugin_r.yaml")
 
     sconfig = ConfigReader._load(format="yaml", content=config_plugins)
@@ -1293,7 +1442,9 @@ def test_load_attached_within_tmux_detached(
     assert switch_client_mock.call_count == 1
 
 
-def test_load_append_windows_to_current_session(server, monkeypatch):
+def test_load_append_windows_to_current_session(
+    server: "Server", monkeypatch: pytest.MonkeyPatch
+) -> None:
     yaml_config = test_utils.read_config_file("workspacebuilder/two_pane.yaml")
     sconfig = ConfigReader._load(format="yaml", content=yaml_config)
 
@@ -1313,11 +1464,15 @@ def test_load_append_windows_to_current_session(server, monkeypatch):
     assert len(server._list_windows()) == 6
 
 
-def test_debug_info_cli(monkeypatch, tmp_path: pathlib.Path):
+def test_debug_info_cli(
+    monkeypatch: pytest.MonkeyPatch,
+    tmp_path: pathlib.Path,
+    capsys: pytest.CaptureFixture,
+) -> None:
     monkeypatch.setenv("SHELL", "/bin/bash")
 
-    runner = CliRunner()
-    cli_output = runner.invoke(command_debug_info).output
+    cli.cli(["debug-info"])
+    cli_output = capsys.readouterr().out
     assert "environment" in cli_output
     assert "python version" in cli_output
     assert "system PATH" in cli_output
